{$A-,B-,D+,E-,F-,I-,L+,N-,O-,R-,S+,V-}
{ Pascal implementation. Code File Handler. (c) 1989 YGP }

unit CFH; interface
  uses Machine;

  type alpha = packed array [0..7] of char;

       DateRecord = record
	 Day: 1..31;
	 Month: 0..11;
	 Year: 1980..2099;
       end; { DateRecord }

       TimeRecord = record
	 Hours: 0..23;
	 Minutes, Seconds: 0..59;
       end; { TimeRecord }

       ModInfo = record
	 name: alpha; {      8 }
	 version: Cardinal;
	 Date: DateRecord;
	 Time: TimeRecord;
       end; { ModInfo }

  type ExitCode = (OK,
		   NoMore, NotCode, InvWrdSz, InvBsex,
		   Para2large, Block2large,
		   MDout, SDout, NotMatch,
		   IOerr);

       ModuleDescriptor = record
	 Mname: alpha;			       { Module name }
	 Version:  Cardinal;		       { Module Version }
	 CDate, 			       { Date and }
	 CTime:    Cardinal;		       {	  Time of compilation }
	 NextMod:  Cardinal;		       { Next Module Descriptor }
	 MainSeg:  Cardinal;		       { Paragraph of Main Segment }
	 Language: Cardinal;		       { Language }
	 CompStart,			       { Start of compiler info }
	 CompSize: Cardinal;		       { Compiler Size }
	 ExtrnList,			       { External List }
	 NExtrnl:  Cardinal;		       { Number of External Segs }
       end; { ModuleDescriptor }

       SegmentDescriptor = record
	 SName: alpha;
	 Start,
	 Size: Cardinal;
	 SegProc: Cardinal;
	 NextSeg: Cardinal;
	 Nunits: Cardinal;
       end; { SegmentDescriptor }

       F_N_Buffer = record
	 f:	    ^file;   {    }
	 ParaSize,
	 BlockSize: Cardinal;
	 Next:	    Cardinal; {      }
	 NextSeg:   Cardinal;
	 Extrnl:    Cardinal; { External List paragraph }
	 CurrMD:    ModuleDescriptor;
	 CurrSD:    SegmentDescriptor;
       end; { F_N_Buffer }

       Segment = record
	 Check: alpha;
	 Wsz, Bsex: byte;
	 VerMaj, VerMin: byte;
	 TotSz: word;
	 CpoolSz: word;
	 unused: array [0..11] of byte;
	 PDict: word;
	 Cpool: word;
	 code: packed array [0..32000] of byte;
       end; { Segment }

       SegmentP = ^Segment;

  var ErrorNo: word;

  procedure GetFirstMod(var f: file; var FNB: F_N_Buffer; var Info: ModInfo;
			var Status: ExitCode);
  procedure GetNextMod(var FNB: F_N_Buffer; var Info: ModInfo;
		       var Status: ExitCode);

  procedure GetNextSegment(var FNB: F_N_Buffer; var Status: ExitCode);

  procedure GetText(Code: ExitCode; var s: string);

  procedure ReadInSeg(FNB: F_N_Buffer; buff: SegmentP; Status: ExitCode);

implementation

  type CodeFileHeader = record
	 CODE:		word;	{ $C0DE }
	 Wsz, Bsex:	byte;	{ 16, 0 }
	 VMaj, VMin:	byte;	{ 0.10	}
	 Unused:	word;
	 ParaSize:	word;	{ Paragraph size in bytes }
	 BlockSize:	word;	{ Block size in bytes }
	 Offset:	word;	{ Offset to first module }
       end; { CodeFileHeader }

  procedure GetFirstMod(var f: file; var FNB: F_N_Buffer; var Info: ModInfo;
			var Status: ExitCode);
    var CFH: CodeFileHeader;
        flmd: byte;
  begin
  Status:= IOerr;
  { Initialize FNBuffer }
  FillChar(FNB, SizeOf(FNB), 0);
  FNB.f:= addr(f);

  close(f); if IOresult <> 0 then;
  flmd:= filemode; FileMode:= 0;
  reset(f, 1); ErrorNo:= IOresult;
  FileMode:= flmd;
  if ErrorNo <> 0 then exit;

  BlockRead(f, CFH, SizeOf(CFH));
  ErrorNo:= IOresult;
  if ErrorNo <> 0 then exit;

  if CFH.CODE <> $DEC0
    then begin Status:= NotCode;
	 exit;
	 end;

  if CFH.Wsz <> 16
    then begin Status:= InvWrdSz;
	 exit;
	 end;

  if CFH.Bsex <> 0
    then begin Status:= InvBsex;
	 exit;
	 end;

  if CFH.ParaSize > 512
    then begin Status:= Para2large;
	 exit;
	 end;

  if CFH.BlockSize > 1024
    then begin Status:= Block2large;
	 exit;
	 end;

  FNB.ParaSize:= CFH.ParaSize;
  FNB.BlockSize:= CFH.BlockSize;

  if FileSize(f) < LongInt(CFH.Offset * FNB.ParaSize)
    then begin Status:= MDout;
	 exit;
	 end;

  FNB.Next:= CFH.Offset;

  GetNextMod(FNB, Info, Status);
  end; { GetFirstMod }

  procedure GetNextMod(var FNB: F_N_Buffer; var Info: ModInfo;
		       var Status: ExitCode);
    var d, t: cardinal;
	MD: ModuleDescriptor;
  begin
  Status:= IOerr;

  if FNB.Next = 0
    then begin Status:= NoMore;
	 exit;
	 end;
  Seek(FNB.f^, LongInt(FNB.Next * FNB.ParaSize));
  ErrorNo:= IOresult; if ErrorNo <> 0 then exit;

  BlockRead(FNB.f^, MD, SizeOf(MD));
  ErrorNo:= IOresult; if ErrorNo <> 0 then exit;

  FNB.Next:=	MD.NextMod;
  FNB.NextSeg:= MD.MainSeg;

  Status:= OK;

  with Info do
    begin name:= MD.Mname;
    version:=	 MD.version;

    d:= 	 swap(MD.CDate);
    t:= 	 swap(MD.CTime);

    Date.Day:=	 d mod 32;
    Date.Month:= (d div 32) mod 16 - 1;
    Date.Year:=  1980 + d div $200;

    Time.Seconds:= (t mod 32) * 2;
    Time.Minutes:= (t div 32) mod 64;
    Time.Hours:=   t div $800;
    end; { with Info }

  FNB.CurrMD:= MD;
  end; { GetNextMod }

  procedure GetNextSegment(var FNB: F_N_Buffer; var Status: ExitCode);
    var d, t: cardinal;
	SD: SegmentDescriptor;
  begin
  Status:= IOerr;

  if FNB.NextSeg = 0
    then begin Status:= NoMore; exit; end;

  Seek(FNB.f^, LongInt(FNB.NextSeg * FNB.ParaSize));
  ErrorNo:= IOresult; if ErrorNo <> 0 then exit;

  BlockRead(FNB.f^, SD, SizeOf(SD));
  ErrorNo:= IOresult; if ErrorNo <> 0 then exit;

  FNB.NextSeg:= SD.NextSeg;

  Status:= OK;

  FNB.CurrSD:= SD;
  end; { GetNextSegment }

  procedure ReadInSeg(FNB: F_N_Buffer; buff: SegmentP; Status: ExitCode);
  begin
  Seek(FNB.f^, LongInt(FNB.CurrSD.Start * FNB.BlockSize));
  BlockRead(FNB.f^, buff^, FNB.CurrSD.Size);
  Status:= IOerr;
  ErrorNo:= IOresult; if ErrorNo <> 0 then exit;
  Status:= OK;

  if buff^.check <> FNB.CurrSD.SName
    then begin Status:= NotMatch; exit; end;

  if buff^.Wsz <> 16
    then begin Status:= InvWrdSz; exit; end;

  if buff^.Bsex <> 0
    then begin Status:= InvBsex; exit; end;

  end; { ReadInSeg }

  procedure GetText(Code: ExitCode; var s: string);
  begin
  case Code of
    OK: 	 s:= 'No problems';
    NoMore:	 s:= 'No more modules';
    NotCode:	 s:= 'Not a Pascal Y codefile';
    InvWrdSz:	 s:= 'Invalid word size';
    InvBsex:	 s:= 'Invalid Byte Sex';
    Para2large:  s:= 'Paragraph size too large';
    Block2large: s:= 'Block size too large';
    MDout:	 s:= 'Module descriptor out of file';
    SDout:	 s:= 'Segment descriptor out of file';
    IOerr:	 begin Str(ErrorNo, s);
		 s:= 'IOerror: ' + s;
		 end;
    else	 begin Str(ord(Code): 5, s);
		 s:= '   No ' + s;
		 end;
    end; { case }
  end; { GetText }

begin
ErrorNo:= 0;
end. { CFH }
